"""
For reference, see the [well section of the OME-Zarr specification](https://ngff.openmicroscopy.org/0.4/#well-md).
"""

from ome_zarr_models.common.well_types import WellImage, WellMeta

__all__ = ["WellImage", "WellMeta"]
